use std::{fs, path::PathBuf};

use sqlx::{migrate::MigrateDatabase, Connection, Sqlite, SqliteConnection, SqlitePool};

#[derive(sqlx::FromRow, Debug)]
pub struct Sheet {
    name: String,
    // #[sqlx(from = "String")]
    path: String,
    // #[sqlx(from = "i64")]
    file_size: i32,
    file_hash: String,
}

#[derive(sqlx::FromRow)]
pub struct Composer {
    name: String,
    id: u32,
}

impl Sheet {
    pub fn new_debug() -> Self {
        Sheet {
            name: "Hello world".to_string(),
            path: "This/is/my/path".into(),
            file_size: 42,
            file_hash: "h4sh".to_string(),
        }
    }

    pub fn verify_path(&self) -> std::io::Result<bool> {
        // First compare file size since it is faster than hashing
        // let file_size = fs::metadata(&self.path)?.len();
        // if file_size == self.file_size {
        //     let file_content = fs::read(&self.path)?;
        //     let file_hash = blake3::hash(&file_content);
        //     if file_hash.to_string() == self.file_hash {
        //         return Ok(true);
        //     }
        // }

        Ok(false)
    }
}

pub struct Database {
    connection: Option<SqlitePool>,
}

impl Database {
    pub fn new() -> Self {
        Database { connection: None }
    }

    pub async fn init_connection(&mut self, database_path: &str) -> sqlx::Result<()> {
        if !Sqlite::database_exists(database_path)
            .await
            .unwrap_or(false)
        {
            match Sqlite::create_database(database_path).await {
                Ok(_) => println!("Database created"),
                Err(e) => println!("Error creating database: {}", e),
            }
        } else {
            println!("Database already exists");
        }

        let connection = SqlitePool::connect(database_path).await?;
        self.connection = Some(connection);
        Ok(())
    }

    pub async fn setup_db(&mut self) -> sqlx::Result<()> {
        // TODO: put this in migrations
        if let Some(connection) = &self.connection {
            sqlx::query!("
                CREATE TABLE IF NOT EXISTS sheets (name TEXT, path TEXT, file_size INTEGER, file_hash TEXT);
                CREATE TABLE IF NOT EXISTS composers (name TEXT, id INTEGER);
            ").execute(connection).await?;
        }
        Ok(())
    }

    pub async fn insert_sheet(&mut self, sheet: Sheet) -> sqlx::Result<()> {
        if let Some(connection) = &self.connection {
            // let path = sheet.path.to_str().unwrap().to_string();
            // let size = sheet.file_size as i64;

            let result = sqlx::query!(
                "
                   INSERT INTO sheets (name, path, file_size, file_hash)
                   VALUES ($1, $2, $3, $4)
               ",
                sheet.name,
                sheet.path,
                sheet.file_size,
                sheet.file_hash,
            )
            .execute(connection)
            .await;
            result.unwrap();
            println!("Inserted!!!");
        }
        Ok(())
    }

    pub async fn fetch_all_sheets(&mut self) -> sqlx::Result<Vec<Sheet>> {
        if let Some(connection) = &self.connection {
            // return sqlx::query_as!(Sheet, "SELECT * FROM sheets")
            //     .fetch_all(connection)
            //     .await;
            return sqlx::query_as::<_, Sheet>("SELECT * FROM sheets")
                .fetch_all(connection)
                .await;
            // return sqlx::query_as("SELECT * FROM sheets").fetch_all(connection).await;
        }
        Ok(Vec::new())
    }
}
